/*
  Pm2Wps    : Reads a Program Manager GRP file and create a similar OS/2 WPS folder
  Author    : Paolo Ciccone
  Update    : 29/7/93
  Update    : 31/5/94
 
  $Id: pm2wps.cmd 1.2 1995/03/01 07:21:40 PCICCONE Exp PCICCONE $
*/

Say "PM2WPS - Creates a WPS folder from a Windows PM .GRP File"
Say "Copyright (c) 1993,1994, 1995 by Paolo Ciccone. All rights reserved"

if arg() < 1 then
do
 say
 say "Usage : pm2wps <grpfilename> [-n]"
 exit
end

parse arg fname createSwitch displaySort

nocreate = createSwitch == "-n"                 /* don't create the folder          */
displaySwap = displaySort == "-d"               /* display sort-swapping operations */


/**************************************************
 * size of fundamental structures
 **************************************************/

pointSz = 4
rectSz  = pointSz*2
wordSz  = 2
maxTitleLen = 14

grpHeader. = ""                                 /* Group File header information */

retval = stream( fname, "c", "open read")
if retval <> "READY:" then
do
  say "Could not open file " fname
  exit
end

retval = stream(fName, "C", "close")

grpHeader.id         = charin(fname, 1,4)
grpHeader.checkSum   = charin(fname,,wordSz)
grpHeader.cbGroup    = charin(fname,,wordSz)
grpHeader.cmdShow    = charin(fname,,wordSz)
grpHeader.normRect   = charin(fname,,rectSz)
grpHeader.ptMin      = charin(fname,,pointSz)
grpHeader.Pname      = toNumVal(charin(fname,,wordSz))+1
grpHeader.XPixels    = charin(fname,,wordSz)
grpHeader.YPixels    = charin(fname,,wordSz)
grpHeader.bitsPerPixel = charin(fname,,1)
grpHeader.planes     = charin(fname,,1)
grpHeader.reserved   = charin(fname,,wordSz)
grpHeader.numItems   = toNumVal(charin(fname,,wordSz))

/* reads the array of pointer to item data */

do itemNo = 0 to grpHeader.numItems-1
  grpHeader.itemNo = toNumVal(charIn(fname,,wordSz))+1
end

folderName = read_asciiz(fname,grpHeader.PName)

say
say "         ID : " grpHeader.id
say "Group Title : " folderName
say "      Items : " grpHeader.numItems
say
say "*** Items ***"

items.0 = -1                                            /* stem containing the array of items */
do itemNo = 0 to grpHeader.numItems-1
  grpItem. = ""
  if grpHeader.itemNo > 1 then
  do
    grpItem. = ""                                       /* temporary used by getItemData */
    call getItemData grpHeader.itemNo
    items.itemNo.point.x = grpItem.point.x
    items.itemNo.point.y = grpItem.point.y
    items.itemNo.title   = grpItem.title
    items.itemNo.cmdLine = grpItem.cmdLine
    items.itemNo.Icon    = grpItem.icon
    items.0 = items.0+1
  end
end

call bubbleSort                                         /* reorder the array so the icons will be in the same */ 
                                                        /* order of the original P.M. group                   */
do itemNo = items.0 to 0 by -1                          /* display the information                            */
  say left(items.itemNo.title,25) " : " items.itemNo.cmdLine
end

if nocreate then
  exit 0


say "Creating folder..."
call startLib

folderID = createFolder( folderName )

do itemNo = 0 to items.0
  call createIconObject items.itemNo.title, items.itemNo.cmdLine, folderId, items.itemNo.icon
  call charout ,"."
end
say "Done !"

exit 0


/* Inserts a caret (^) in the midle of the icon title to split long title in two lines */
/* A title is considered "long" if its lenght is greater then the value stored in the  */
/* maxTitleLen variable                                                                */
insertCaret : procedure expose maxTitleLen
  title = arg(1)
  sepPos = pos(' ', title, maxTitleLen )
  if ( sepPos > 0 ) then
    title = substr(title,1, sepPos ) || "^" || substr(title,sepPos+1)
return title




/* Converts a two-byte character array into a smallInt variable */
toNumVal: procedure
  numString = arg(1)
return C2D(left(numstring,1))+c2d(right(numstring,1))*256


/* Reads item data from the group file */
getItemData : procedure expose wordSz pointSz fSize fName grpItem.
  itemDataOfs = arg(1)
  itemData. = ""

  itemData.point.x   = toNumVal(charin(fname,itemDataOfs,wordSz))
  itemData.point.y   = toNumVal(charin(fname,,wordSz))
  itemData.iconNum   = toNumVal(charin(fname,,wordSz))
  itemData.iconsz    = charin(fname,,wordSz)
  itemData.cAndMask  = charin(fname,,wordSz)
  itemData.cXorMask  = charin(fname,,wordSz)
  itemData.iHeader   = charin(fname,,wordSz)
  itemData.andMask   = charin(fname,,wordSz)
  itemData.xorMask   = charin(fname,,wordSz)
  itemData.PName     = toNumVal(charin(fname,,wordSz))+1
  itemData.PCmdLine  = toNumVAl(charin(fname,,wordSz))+1
  itemData.PIconPath = toNumVal(charin(fname,,wordSz))+1
  grpItem. = ""
  grpItem.point.x = itemData.point.x
  grpItem.point.y = itemData.point.y
  grpItem.title   = read_asciiz(fname, itemData.PName)
  grpItem.cmdLine = read_asciiz(fname, itemData.PcmdLine)
  grpItem.Icon    = read_asciiz(fname, itemData.pIconPath)
return


/* Calls the REXX utility functions */
startLib : procedure
  call rxFuncAdd sysLoadFuncs, rexxUtil, sysLoadFuncs
  call sysLoadFuncs
return


/* Creates the program object in the folder */
createIconObject : procedure expose maxTitleLen
  title    = arg(1)
  cmdLine  = arg(2)
  folderId = arg(3)
  iconFile = arg(4)
  if ( length(title) > maxTitleLen )  then
    title = insertCaret( title )
  parse var cmdLine appName parameters
  dotPos = pos( ".", appName )
  if ( dotPos > 0 )  then
  do
    extension = substr( appName, dotPos+1, 3)
  end
  if ( translate(extension) =  "HLP") then
    do
      parameters = appname || " " || parameters
      appName = "winhelp.exe"
      eType = "Windows"
    end

  appname = translate(appname)
  if  appname == "WINHELP.EXE" then
    eType = "Windows"
  else
    eType = exeType(appname)

  select
    when eType = "Windows" then
      progType = "PROG_31_ENHSEAMLESSCOMMON"
    otherwise
      progType = "WINDOWEDVDM"
  end   
  setup = "EXENAME=" || appNAme || ";PARAMETERS=" || parameters
  setup = setup || ";PROGTYPE=" || progType 
  if ( \ (iconFile == cmdLine) ) then
    setup = setup || ";ICONFILE=" || iconFile
  retVal = buildObject( "WPProgram", title, folderID, setup )
return


/* Creates the WPS folder. Default to FLOWED Icon View */
createFolder : procedure
  location  = "<WP_DESKTOP>"
  objId = "<" || translate(arg(1),"_"," ") || ">"
  setup     = "OBJECTID=" || objId || ";"
  call buildObject "WPFolder", arg(1), location, setup || "OPEN=ICON;ICONVIEW=FLOWED;ICONFONT=8.Helv" 
return objId


/* Low-level function to create WPS object. Called by createFolder and createIconObject */
buildObject : procedure
  retval = sysCreateObject( arg(1), arg(2), arg(3),arg(4) )
  if \ retval == 1 then
    do
       say "Error building object " arg(2) " : code " retval
       say "classname : " arg(1) " location : " arg(3)
       say "Setup :" arg(4)
    end
return retval


/* Reads an ASCIIZ string from a stream */
read_asciiz : procedure
  fname = arg(1)        /* stream */
  f_seek = arg(2)
  if f_seek > 0  then
    call charin fname,f_seek, 0
  str = ""
  do forever
    ch = charin( fname,,1 )
    if c2d(ch) == 0 then
      leave
    else
      str = str || ch
  end /* do */
return str


/* Swaps two item's data. Used by procedure bubbleSort to swap elements of the array */
swapItem : procedure expose items. displaySwap
 theIndex = arg(1)
 p = theIndex-1
 if displaySwap then
   say "swapping " items.theIndex.title " with " items.p.title 
 temp. = ""
 temp.point.x = items.p.point.x
 temp.point.y = items.p.point.y
 temp.title   = items.p.title
 temp.cmdLine = items.p.cmdLine
 temp.icon    = items.p.icon

 items.p.point.x = items.theIndex.point.x
 items.p.point.y = items.theIndex.point.y
 items.p.title   = items.theIndex.title
 items.p.cmdLine = items.theIndex.cmdLine
 items.p.icon    = items.theIndex.icon

 items.theIndex.point.x = temp.point.x
 items.theIndex.point.y = temp.point.y
 items.theIndex.title   = temp.title
 items.theIndex.cmdLine = temp.cmdLine
 items.theIndex.icon    = temp.icon
return


/* Sorts the array of items based by the coordinates.The resulting array is an ordered */
/* collection of rows. Every row is an ordered collection of icons. Due to a quirk in  */
/* the way the WPS order the elements in a folder, the array is sorted in reverse order*/
/* the first element is the last icon of the original PM group. This trick plus the    */
/* flowed folder,creates a set of icons ordered in the rights way but flowing from top */
/* to bottom instead of left to right                                                  */
bubbleSort : procedure expose items. displaySwap
  do i=1 to items.0
     do j=items.0 to i by -1
       p = j-1
       pt1 = items.p.point.y * 100000 + items.p.point.x
       pt2 = items.j.point.y * 100000 + items.j.point.x
       if ( pt1 < pt2 )  then
         call swapItem j
     end /* do */
  end
return


/* Returns a string describing the type of executable ( DOS, OS/2, Windows ... ) */
exeType : procedure

  fname = arg(1)

  eType = "Error"
  signature = charin(fname,,2)
  if ( signature = "MZ" | signature = "ZM" ) then
    do
      exeType1 = toNumVal(charin( fname, 25, 2 ))
      if ( exeType1 >= 64 ) then
        do
          ne = toNumVal( charin( fname, 61, 2 ) )+1
          neSig = charin( fname, ne, 2 )
          select
             when neSig = "NE" then
                do
                  osType = C2D(charin( fname , ne+54, 1 ))
                  select
                     when osType = 1 then
                        eType = "OS/2 1"
                     when osType = 2 then
                        eType = "Windows"
                  otherwise
                     eType = "Special"
                  end  /* select */
                end /* do */
             when neSig = "LE" then
                eType =  "VxD"
             when neSig = "LX" then
                eType =  "OS/2"
             when neSig = "W3" then
                eType = "WN386.EXE"
             when neSig = "PE" then
                eType = "Win32"
          otherwise
            eType = "Unknown"
          end  /* select */
        end
      else
        eType = "DOS"
    end
  else
    eType = "Error"
  call stream fname, "C", "CLOSE"
return eType


